local Tinkr, Bastion = ...
local Interface = {}

--[[
    The current settings is named "Vertical" as it uses the Dragonflight Vertical
    layout, rather than the Canvas layout. Vertical looks better, is easier to
    format and use. Canvas layout allows you to add multi-select dropdowns, text
    inputs, and more. I prefer vertical. Canvas support will be added in the future.
]]--

local Hotbar = {}
Hotbar.__index = Hotbar

Bastion.Hotbars = {}

-- Constructor
---@return Hotbar
function Hotbar:New(details)
    local self = setmetatable({}, Hotbar)
    self.button_size        = 45
    self.button_spacing     = 5
    self.button_count       = details.buttonCount
    self.name               = details.name
    self.options            = details.options

    local position = self.options.config:Read("bar_location", { "CENTER", 0, -50 })


        self.frame = CreateFrame("Frame", nil, UIParent, "SecureHandlerStateTemplate")
        self.frame:SetWidth(self.button_count * (32 + self.button_spacing) - self.button_spacing)  -- width = total width of all buttons + total width of all spacings
        self.frame:SetHeight(32 + 20)


    self.frame:SetPoint(position[1], position[2], position[3])

    self.title = self.frame:CreateFontString(nil, "OVERLAY", "Game15Font")

        self.title:SetPoint("BOTTOMLEFT", self.frame, "LEFT", 0, 10)  -- adjust these values as necessary

    self.title:SetText(self.name)  -- replace this with your desired title

    self.frame:EnableMouse(true)
    self.frame:SetMovable(true)
    self.frame:SetClampedToScreen(true)
    self.frame:RegisterForDrag("LeftButton")
    self.frame:SetScript("OnDragStart", self.frame.StartMoving)
    self.frame:SetScript("OnDragStop", function()
        self.frame:StopMovingOrSizing()
        local point, _, _, x, y = self.frame:GetPoint()
        self.options.config:Write("bar_location", { point, x, y })
    end)

    if not self.options.config:Read("display_hotbar_" .. self.options.name, true) then
        self.frame:Hide()
    end

    self.options:Checkbox({
        category = "display_hotbar",
        var = self.name,
        name = self.name .. " Hotbar",
        tooltip = "Display the hotbar for " .. self.name,
        default = true
    })

    self.buttons = {}

    table.insert(Bastion.Hotbars, self)
    return self
end

-- Needed to ensure hotbars are positioned properly and hidden when a user hides them in settings
function Hotbar:Refresh()
    if not self.options.config:Read("display_hotbar_" .. self.options.name, true) then
        if self.frame:IsVisible() then
            self.frame:Hide()
        end
    else
        if not self.frame:IsVisible() then
            self.frame:Show()
        end
    end

    return self
end

-- Adds a button to the hotbar.
-- TODO: Make this not a hot mess for classic/retail
function Hotbar:AddButton(details)
        local button = CreateFrame("CheckButton", nil, self.frame, "UIPanelButtonTemplate")
        button:SetNormalTexture(details.texture)
        button:SetPoint("CENTER")
        button:SetSize(32, 32)
        table.insert(self.buttons, button)
        if details.onClick then
            if details.toggle then
                local toggleAnim = false
                button:SetScript("OnClick", function()
                    -- Toggle animation
                    if toggleAnim then
                        ActionButton_HideOverlayGlow(button)
                        toggleAnim = false
                    else
                        ActionButton_ShowOverlayGlow(button)
                        toggleAnim = true
                    end
                    details.onClick()
                end)
            else
                button:SetScript("OnClick", details.onClick)
            end
        end
        button:SetScript("OnEnter", function(self)
            GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
            GameTooltip:AddLine(details.name, 1, 1, 1)  -- sets the tooltip as the button's name
            GameTooltip:AddLine(details.tooltip)
            GameTooltip:Show()
        end)
        button:SetScript("OnLeave", function(self)
            GameTooltip:Hide()
        end)
        for i, btn in ipairs(self.buttons) do
            button:SetPoint("BOTTOMLEFT", (i - 1) * (32 + self.button_spacing), 0)  -- Position each button
        end

        return self
end

local Category = {}
Category.__index = Category

-- Constructor
---@param name String
---@param config Config
---@return Category
function Category:New(name, parent)
    local self = setmetatable({}, Category)
    self.name = name
    self.parent = parent or nil

    if self.parent then
        self.settings, self.layout = Settings.RegisterVerticalLayoutSubcategory(parent.settings, self.name)
        self.config = self.parent.config
    else
        self.settings, self.layout = Settings.RegisterVerticalLayoutCategory(self.name)
        self.config = Bastion.Config:New(name)
    end

    return self
end

function Category:AddSubsection(section_title)
    local section = CreateSettingsListSectionHeaderInitializer(section_title)
    self.layout:AddInitializer(section)
    return self
end

-- Register the Category object with the Blizzard UI
---@return nil
function Category:Register()
    Settings.RegisterAddOnCategory(self.settings)
end

-- Adds a checkbox to the Category
---@param details Table
function Category:Checkbox(details)
    local default_value = details.default or false
    local variable_name = details.category .. "_" .. details.var
    
    local setting = Settings.RegisterAddOnSetting(self.settings, details.name, variable_name, type(default_value), self.config:Read(variable_name, default_value))

    Settings.SetOnValueChangedCallback(variable_name, function(event)
        self.config:Write(variable_name, setting:GetValue())
    end)

    local initializer = Settings.CreateCheckBox(self.settings, setting, details.tooltip and ("\n" .. details.tooltip) or "")

    if details.disabled then
        initializer:AddModifyPredicate(function() return false end)
        self.config:Write(variable_name, false)
    end

    return self
end

-- Adds a checkbox to the Category
---@param details Table
function Category:SubCheck(details)
    local default_value = details.default or false
    local variable_name = details.category .. "_" .. details.var
    
    local setting = Settings.RegisterAddOnSetting(self.settings, details.name, variable_name, type(default_value), self.config:Read(variable_name, default_value))

    Settings.SetOnValueChangedCallback(variable_name, function(event)
		print(variable_name, setting:GetValue())
        self.config:Write(variable_name, setting:GetValue())
        details.sub:Toggle()
    end)

    local initializer = Settings.CreateCheckBox(self.settings, setting, details.tooltip and ("\n" .. details.tooltip) or "")

    return self
end

-- Adds a slider to the Category
---@param details Table
function Category:Slider(details)
    details = details or {} 
    local default_value = details.default or 0
    local min_value = details.min or 0
    local max_value = details.max or 100

    local step = details.step or 1
    local variable_name = details.category .. "_" .. details.var

    local setting = Settings.RegisterAddOnSetting(self.settings, details.name, variable_name, type(default_value), tonumber(self.config:Read(variable_name, default_value)))

    Settings.SetOnValueChangedCallback(variable_name, function(event)
        self.config:Write(variable_name, setting:GetValue())
    end)

    local options = Settings.CreateSliderOptions(min_value, max_value, step)

    options:SetLabelFormatter(MinimalSliderWithSteppersMixin.Label.Right)

    local initializer = Settings.CreateSlider(self.settings, setting, options, details.tooltip and ("\n" .. details.tooltip) or "")

    -- if details.disabled then
    --     initializer:AddModifyPredicate(function() return false end)
    --     self.config:Write(variable_name, false)
    -- end

    return self
end

-- Adds a dropdown to the category
---@param details Table
function Category:Dropdown(details)
    local default_value = details.default
    local variable_name = details.category .. "_" .. details.var

    local setting = Settings.RegisterAddOnSetting(self.settings, details.name, variable_name, type(default_value), self.config:Read(variable_name, default_value))

    local function GetOptions()
        local container = Settings.CreateControlTextContainer()
        for k, v in ipairs(details.options) do 
            container:Add(v[2], v[1])
        end
        return container:GetData()
    end

    Settings.SetOnValueChangedCallback(variable_name, function(event)
        self.config:Write(variable_name, setting:GetValue())
    end)

    local initializer = Settings.CreateDropDown(self.settings, setting, GetOptions, details.tooltip or "")

    if details.disabled then
        initializer:AddModifyPredicate(function() return false end)
        self.config:Write(variable_name, false)
    end

    return self
end

Interface.Hotbar = Hotbar
Interface.Category = Category
print("Interface Loaded")
Bastion.Interface = Interface